package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.QuickReplyTypeEnum;
import com.gic.evaluate.constant.TrendDateUnitEnum;
import com.gic.evaluate.qo.StoreScoreQO;
import com.gic.evaluate.qo.StoreTrendQO;
import com.gic.evaluate.service.EvaluateOverviewApiService;
import com.gic.evaluate.service.EvaluateService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-27 16:52
 */
@Service("evaluateOverviewApiService")
public class EvaluateOverviewApiServiceImpl implements EvaluateOverviewApiService {

    private Logger logger = LogManager.getLogger(EvaluateOverviewApiServiceImpl.class);

    @Autowired
    private EvaluateService evaluateService;

    @Override
    public ServiceResponse<Map<String, Long>> chart1(Integer enterpriseId, String startTime, String endTime) {
        Map<String, Long> countMap = evaluateService.getTotalCountChart(enterpriseId, startTime, endTime);
        return EnterpriseServiceResponse.success(countMap);
    }

    @Override
    public ServiceResponse<HashMap<String, Map<String, Long>>> chart2(Integer enterpriseId, String startTime, String endTime) {
        Map<String, Long> goodsCountMap = evaluateService.getGoodsCountChart(enterpriseId, startTime, endTime);
        Map<String, Long> storeCountMap = evaluateService.getStoreCountChart(enterpriseId, startTime, endTime);
        Map<String, Long> serviceCountMap = evaluateService.getServiceCountChart(enterpriseId, startTime, endTime);

        HashMap<String, Map<String, Long>> hashMap = new HashMap<>(4);
        hashMap.put("goodsCountMap", goodsCountMap);
        hashMap.put("storeCountMap", storeCountMap);
        hashMap.put("serviceCountMap", serviceCountMap);


        return EnterpriseServiceResponse.success(hashMap);
    }

    @Override
    public ServiceResponse<Map<String, List<Map<String, Object>>>> chart3(Integer enterpriseId, String startTime, String endTime) {
        List<Map<String, Object>> listTrendTotalEvaluate = this.evaluateService.listTrendTotalEvaluate(enterpriseId, startTime, endTime);
        List<Map<String, Object>> listTrendGoodEvaluate = this.evaluateService.listTrendGoodEvaluate(enterpriseId, startTime, endTime);
        // todo order trend
        Map<String, List<Map<String, Object>>> map = new HashMap<>();


        try {
            List<String> dateList = this.getDateList(startTime, endTime);

            map.put("totalEvaluateList", dealChart4(listTrendTotalEvaluate, dateList));
            map.put("goodEvaluateList", dealChart4(listTrendGoodEvaluate, dateList));
            map.put("orderList", new ArrayList<>());
            return EnterpriseServiceResponse.success(map);
        } catch (Exception e) {
            logger.info("解析参数错误", e);
            return EnterpriseServiceResponse.failure(ErrorCode.SYSTEM_ERROR);
        }
    }

    @Override
    public ServiceResponse<Map<String, List<Map<String, Object>>>> chart4(Integer enterpriseId, String startTime, String endTime) {
        List<Map<String, Object>> goodChart = this.evaluateService.evaluateGoodQuickChart(enterpriseId, startTime, endTime);
        List<Map<String, Object>> badChart = this.evaluateService.evaluateBadQuickChart(enterpriseId, startTime, endTime);

        Map<String, List<Map<String, Object>>> map = new HashMap<>(4);
        map.put("goodChart", goodChart);
        map.put("badChart", badChart);
        return EnterpriseServiceResponse.success(map);
    }

    private List<Map<String, Object>> dealChart4(List<Map<String, Object>> chart, List<String> dateList) {
        List<Map<String, Object>> result = new ArrayList<>();
        for (String date : dateList) {
            Map<Object, Object> collect = chart.stream().collect(Collectors.toMap(e -> e.get("statisticsDate"), e -> e.get("count")));
            Object value = collect.get(date);
            Map<String, Object> map = new HashMap<>(4);
            map.put(date, value == null ? 0 : value);
            result.add(map);
        }
        return result;
    }

    @Override
    public ServiceResponse<List<Map<String, Object>>> listStoreScore(StoreScoreQO storeScoreQO) {
        List<Map<String, Object>> storeScoreList = this.evaluateService.listStoreScore(storeScoreQO);
        return EnterpriseServiceResponse.success(storeScoreList);
    }

    @Override
    public ServiceResponse<Map<String, Object>> getTotalStoreScore(StoreScoreQO storeScoreQO) {
        Map<String, Object> storeScore = this.evaluateService.getTotalStoreScore(storeScoreQO);
        return EnterpriseServiceResponse.success(storeScore);
    }

    @Override
    public ServiceResponse<List<Map<String, Object>>> listTrend(StoreTrendQO storeTrendQO) {
        List<Map<String, Object>> trendList = this.evaluateService.listTrend(storeTrendQO);
        if (CollectionUtils.isNotEmpty(trendList)) {
            List<Integer> storeIdList = trendList.stream().map(
                    e -> {
                        Object storeId = e.get("storeId");
                        return storeId == null ? null : Integer.valueOf(storeId.toString());
                    }
            ).collect(Collectors.toList());
            // todo order  通过storeIdList拿到order总数
        }
        return EnterpriseServiceResponse.success(trendList);
    }

    @Override
    public ServiceResponse<List<Map<String, Object>>> listGoodEvaluateTrendByDate(Integer enterpriseId, Integer type) {
        if (TrendDateUnitEnum.YEAR.getCode().equals(type)) {
            // 按年
            List<Map<String, Object>> list = evaluateService.listEvaluateTrendByYear(enterpriseId, QuickReplyTypeEnum.GOOD.getCode());
            return EnterpriseServiceResponse.success(list);
        } else if (TrendDateUnitEnum.MONTH.getCode().equals(type)) {
            // 按月
            List<Map<String, Object>> list = evaluateService.listEvaluateTrendByMonth(enterpriseId, QuickReplyTypeEnum.GOOD.getCode());
            return EnterpriseServiceResponse.success(list);
        } else if (TrendDateUnitEnum.DAY.getCode().equals(type)) {
            // 按日
            List<Map<String, Object>> list = evaluateService.listEvaluateTrendByDay(enterpriseId, QuickReplyTypeEnum.GOOD.getCode());
            return EnterpriseServiceResponse.success(list);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
    }

    @Override
    public ServiceResponse<List<Map<String, Object>>> listBadEvaluateTrendByDate(Integer enterpriseId, Integer type) {
        if (TrendDateUnitEnum.YEAR.getCode().equals(type)) {
            // 按年
            List<Map<String, Object>> list = evaluateService.listEvaluateTrendByYear(enterpriseId, QuickReplyTypeEnum.BAD.getCode());
            return EnterpriseServiceResponse.success(list);
        } else if (TrendDateUnitEnum.MONTH.getCode().equals(type)) {
            // 按月
            List<Map<String, Object>> list = evaluateService.listEvaluateTrendByMonth(enterpriseId, QuickReplyTypeEnum.BAD.getCode());
            return EnterpriseServiceResponse.success(list);
        } else if (TrendDateUnitEnum.DAY.getCode().equals(type)) {
            // 按日
            List<Map<String, Object>> list = evaluateService.listEvaluateTrendByDay(enterpriseId, QuickReplyTypeEnum.BAD.getCode());
            return EnterpriseServiceResponse.success(list);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
    }

    private List<String> getDateList(String dateFirst, String dateSecond) throws ParseException {
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
        Date dateOne = dateFormat.parse(dateFirst);
        Date dateTwo = dateFormat.parse(dateSecond);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(dateOne);

        List<String> dateList = new ArrayList<>();
        while (calendar.getTime().before(dateTwo)) {
            dateList.add(dateFormat.format(calendar.getTime()));
            calendar.add(Calendar.DAY_OF_MONTH, 1);
        }
        dateList.add(dateSecond);
        return dateList;
    }

}
