package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.dto.ProblemReplyDTO;
import com.gic.evaluate.dto.ProblemUserReplyDTO;
import com.gic.evaluate.entity.TabProblem;
import com.gic.evaluate.entity.TabProblemImg;
import com.gic.evaluate.qo.ProblemQO;
import com.gic.evaluate.service.ProblemImgService;
import com.gic.evaluate.service.ProblemManageApiService;
import com.gic.evaluate.service.ProblemReplyService;
import com.gic.evaluate.service.ProblemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 11:08
 */
@Service("problemManageApiService")
public class ProblemManageApiServiceImpl implements ProblemManageApiService {

    @Autowired
    private ProblemService problemService;
    @Autowired
    private ProblemImgService problemImgService;
    @Autowired
    private ProblemReplyService problemReplyService;

    @Override
    public ServiceResponse<List<ProblemDTO>> listProblem(ProblemQO problemQO) {
        List<TabProblem> problemList = problemService.listProblem(problemQO);
        List<ProblemDTO> dtoList = EntityUtil.changeEntityListByJSON(ProblemDTO.class, problemList);
        return EnterpriseServiceResponse.success(dtoList);
    }

    @Override
    public ServiceResponse<ProblemDTO> getProblem(Integer problemId) {
        TabProblem tabProblem = problemService.getProblem(problemId);
        if (tabProblem == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS);
        }
        ProblemDTO problemDTO = EntityUtil.changeEntityByJSON(ProblemDTO.class, tabProblem);
        if (Constants.OPEN.equals(problemDTO.getHasOtherImage())) {
            List<TabProblemImg> tabImgList = problemImgService.listImageByProblemId(problemId);
            List<String> list = Optional.ofNullable(tabImgList).orElse(Collections.emptyList()).stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList());
            problemDTO.setImgList(list);
        }
        List<ProblemReplyDTO> replyList = problemReplyService.listReply(problemId);
        problemDTO.setReplyList(replyList);
        return EnterpriseServiceResponse.success(problemDTO);
    }

    @Override
    public ServiceResponse<Void> replyProblem(ProblemUserReplyDTO problemUserReplyDTO) {
        this.problemReplyService.save(problemUserReplyDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> replyProblem(List<Integer> problemIdList, ProblemUserReplyDTO problemUserReplyDTO) {
        for (Integer problemId : problemIdList) {
            problemUserReplyDTO.setProblemId(problemId);
            this.replyProblem(problemUserReplyDTO);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateMaliciousStatus(Integer problemId, Integer maliciousStatus) {
        ProblemDTO problemDTO = new ProblemDTO();
        problemDTO.setProblemId(problemId);
        problemDTO.setMaliciousStatus(maliciousStatus);
        if (Constants.OPEN.equals(maliciousStatus)) {
            problemDTO.setShowStatus(Constants.CLOSE);
        } else if (Constants.CLOSE.equals(maliciousStatus)) {
            problemDTO.setShowStatus(Constants.OPEN);
        }
        problemService.updateProblem(problemDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> deleteReply(Integer problemReplyId) {
        ProblemReplyDTO problemReplyDTO = new ProblemReplyDTO();
        problemReplyDTO.setProblemReplyId(problemReplyId);
        problemReplyDTO.setStatus(Constants.CLOSE);
        problemReplyService.update(problemReplyDTO);
        return EnterpriseServiceResponse.success();
    }
}
