package com.gic.evaluate.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.evaluate.constant.Constants;
import com.gic.evaluate.dto.ProblemDTO;
import com.gic.evaluate.dto.ProblemMemberReplyDTO;
import com.gic.evaluate.dto.ProblemReplyDTO;
import com.gic.evaluate.entity.TabProblem;
import com.gic.evaluate.entity.TabProblemImg;
import com.gic.evaluate.service.ProblemImgService;
import com.gic.evaluate.service.ProblemOutApiService;
import com.gic.evaluate.service.ProblemReplyService;
import com.gic.evaluate.service.ProblemService;
import com.gic.member.api.dto.MemberUserDTO;
import com.gic.member.api.service.MemberUserApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-01-02 14:31
 */
@Service("problemOutApiService")
public class ProblemOutApiServiceImpl implements ProblemOutApiService {

    private Logger logger = LogManager.getLogger(ProblemOutApiServiceImpl.class);

    @Autowired
    private ProblemService problemService;
    @Autowired
    private ProblemReplyService problemReplyService;
    @Autowired
    private ProblemImgService problemImgService;
    @Autowired
    private MemberUserApiService memberUserApiService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> saveProblem(ProblemDTO problemDTO) {
        logger.info("保存problem:{}", problemDTO);
        List<String> imgList = problemDTO.getImgList();
        if (CollectionUtils.isNotEmpty(imgList)) {
            problemDTO.setImageUrl(imgList.get(0));
            problemDTO.setHasOtherImage(imgList.size() > 1 ? Constants.OPEN : Constants.CLOSE);
        }
        Integer problemId = problemService.save(problemDTO);
        if (CollectionUtils.isNotEmpty(imgList)) {
            problemImgService.saveList(problemDTO.getEnterpriseId(), problemId, imgList);
        }
        ServiceResponse<MemberUserDTO> memberUserDTOServiceResponse = memberUserApiService.getByMcuId(problemDTO.getMemberId(), problemDTO.getEnterpriseId());
        if (memberUserDTOServiceResponse.isSuccess()) {
            MemberUserDTO memberUserDTO = memberUserDTOServiceResponse.getResult();
            problemDTO.setMemberName(memberUserDTO.getName());
            problemDTO.setMemberPhone(memberUserDTO.getMobile());
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveProblemReply(ProblemMemberReplyDTO problemMemberReplyDTO) {
        List<String> imgUrlList = problemMemberReplyDTO.getImgList();
        Integer problemReplyId = problemReplyService.save(problemMemberReplyDTO);
        ServiceResponse<MemberUserDTO> memberUserDTOServiceResponse = memberUserApiService.getByMcuId(problemMemberReplyDTO.getMemberId(), problemMemberReplyDTO.getEnterpriseId());
        if (memberUserDTOServiceResponse.isSuccess()) {
            MemberUserDTO memberUserDTO = memberUserDTOServiceResponse.getResult();
            problemMemberReplyDTO.setMemberName(memberUserDTO.getName());
        }
        if (CollectionUtils.isNotEmpty(imgUrlList)) {
            problemImgService.saveList(problemMemberReplyDTO.getEnterpriseId(), problemMemberReplyDTO.getProblemId(), problemReplyId, imgUrlList);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<List<ProblemDTO>> listProblem(Long memberId) {
        List<ProblemDTO> problemList = problemService.listProblem(memberId);
        if (CollectionUtils.isNotEmpty(problemList)) {
            List<Integer> problemIdList = problemList.stream().map(ProblemDTO::getProblemId).collect(Collectors.toList());
            Map<Integer, String> firstReply = problemReplyService.firstReplyByProblemId(problemIdList);
            for (ProblemDTO problemDTO : problemList) {
                problemDTO.setReplyContent(firstReply.get(problemDTO.getProblemId()));
            }
        }
        return EnterpriseServiceResponse.success(problemList);
    }

    @Override
    public ServiceResponse<ProblemDTO> getProblem(Integer problemId) {
        TabProblem problem = problemService.getProblem(problemId);
        ProblemDTO problemDTO = EntityUtil.changeEntityByJSON(ProblemDTO.class, problem);
        if (Constants.OPEN.equals(problemDTO.getHasOtherImage())) {
            List<TabProblemImg> imgList = problemImgService.listImageByProblemId(problemId);
            List<String> list = Optional.ofNullable(imgList).orElse(Collections.emptyList()).stream().map(TabProblemImg::getImgUrl).collect(Collectors.toList());
            problemDTO.setImgList(list);
        }
        List<ProblemReplyDTO> replyList = problemReplyService.listReply(problemId);
        problemDTO.setReplyList(replyList);
        return EnterpriseServiceResponse.success(problemDTO);
    }
}